@@ -1,14 +1,16 @@
 """
-A collection of 7 ECG heartbeat detection algorithms implemented
+A collection of ECG heartbeat detection algorithms implemented
 in Python. Developed in conjunction with a new ECG database:
-http://researchdata.gla.ac.uk/716/.
+http://researchdata.gla.ac.uk/716/
 
-Copyright (C) 2019 Luis Howell & Bernd Porr
+Copyright (C) 2019-2022 Luis Howell & Bernd Porr
 GPL GNU GENERAL PUBLIC LICENSE Version 3, 29 June 2007
 """
 
 import numpy as np
 import pywt
+import ecgtemplates
+
 try:
     import pathlib
 except ImportError:
@@ -23,16 +25,43 @@ class Detectors:
     The argument ecg_in_samples is a single channel ECG in volt
     at the given sample rate.
     """
-    
-    def __init__(self, sampling_frequency):
+
+    def __init__(self, sampling_frequency = False):
         """
         The constructor takes the sampling rate in Hz of the ECG data.
+        The constructor can be called without speciying a sampling rate to
+        just access the detector_list, however, detection won't
+        be possible.
         """
 
+        ## Sampling rate
         self.fs = sampling_frequency
-        # this is set to a positive value for benchmarking
+
+        ## This is set to a positive value for benchmarking
         self.engzee_fake_delay = 0
 
+        ## 2D Array of the different detectors: [[description,detector]]
+        self.detector_list = [
+            ["Elgendi et al (Two average)",self.two_average_detector],
+            ["Matched filter",self.matched_filter_detector],
+            ["Kalidas & Tamil (Wavelet transform)",self.swt_detector],
+            ["Engzee",self.engzee_detector],
+            ["Christov",self.christov_detector],
+            ["Hamilton",self.hamilton_detector],
+            ["Pan Tompkins",self.pan_tompkins_detector],
+            ["WQRS",self.wqrs_detector]
+        ]
+
+    def get_detector_list(self):
+        """
+        Returns a 2D array of the different detectors in the form:
+        [[description1,detector1],[description2,detector2], ...]
+        where description is a string and detector a function pointer
+        to the detector. Use this for benchmarking to loop through
+        detectors.
+        """
+        return self.detector_list
+
     def hamilton_detector(self, unfiltered_ecg):
         """
         P.S. Hamilton, 
@@ -366,7 +395,7 @@ class Detectors:
         return r_peaks
 
     
-    def matched_filter_detector(self, unfiltered_ecg, template_file = ""):
+    def matched_filter_detector(self, unfiltered_ecg, template_file = False):
         """
         FIR matched filter using template of QRS complex.
         Template provided for 250Hz and 360Hz. Optionally provide your
@@ -375,18 +404,15 @@ class Detectors:
         """
         current_dir = pathlib.Path(__file__).resolve()
 
-        if len(template_file) > 1:
+        if template_file:
             template = np.loadtxt(template_file)
         else:
             if self.fs == 250:
-                template_dir = current_dir.parent/'templates'/'template_250hz.csv'
-                template = np.loadtxt(template_dir)
+                template = ecgtemplates.qrs_250Hz
             elif self.fs == 360:
-                template_dir = current_dir.parent/'templates'/'template_360hz.csv'
-                template = np.loadtxt(template_dir)
+                template = ecgtemplates.qrs_360Hz
             else:
-                print('\n!!No template for this frequency!!\n')
-                return False
+                raise ValueError("!! No stock template for fs = {} !!".format(self.fs))
 
         f0 = 0.1/self.fs
         f1 = 48/self.fs
@@ -406,7 +432,7 @@ class Detectors:
         return squared_peaks
 
     
-    def swt_detector(self, unfiltered_ecg):
+    def swt_detector(self, unfiltered_ecg, MWA_name='cumulative'):
         """
         Stationary Wavelet Transform 
         based on Vignesh Kalidas and Lakshman Tamil. 
@@ -417,6 +443,7 @@ class Detectors:
         Uses the Pan and Tompkins thresolding.
         """
         
+        maxQRSduration = 0.150 #sec
         swt_level=3
         padding = -1
         for i in range(1000):
@@ -435,13 +462,11 @@ class Detectors:
 
         squared = swt_ecg*swt_ecg
 
-        f1 = 0.01/self.fs
-        f2 = 10/self.fs
+        N = int(maxQRSduration*self.fs)
+        mwa = MWA_from_name(MWA_name)(squared, N)
+        mwa[:int(maxQRSduration*self.fs*2)] = 0
 
-        b, a = signal.butter(3, [f1*2, f2*2], btype='bandpass')
-        filtered_squared = signal.lfilter(b, a, squared)       
-
-        filt_peaks = panPeakDetect(filtered_squared, self.fs)
+        filt_peaks = panPeakDetect(mwa, self.fs)
         
         return filt_peaks
 
@@ -454,6 +479,7 @@ class Detectors:
         BME-32.3 (1985), pp. 230–236.
         """
         
+        maxQRSduration = 0.150 #sec
         f1 = 5/self.fs
         f2 = 15/self.fs
 
@@ -465,9 +491,9 @@ class Detectors:
 
         squared = diff*diff
 
-        N = int(0.12*self.fs)
+        N = int(maxQRSduration*self.fs)
         mwa = MWA_from_name(MWA_name)(squared, N)
-        mwa[:int(0.2*self.fs)] = 0
+        mwa[:int(maxQRSduration*self.fs*2)] = 0
 
         mwa_peaks = panPeakDetect(mwa, self.fs)
 
@@ -544,10 +570,11 @@ class Detectors:
         def length_transfrom(x, w):
             tmp = []
             for i in range(w, len(x)):
-                curr = 0
-                for k in range(i-w+1, i):
-                    curr += np.sqrt(np.power(1/self.fs, 2)+np.power(x[i]-x[i-1],2))
-                tmp.append(curr)
+                chunk = x[i-w:i]
+                ll = np.sum(
+                    np.sqrt( np.power(1/self.fs,2)*np.ones(w-1) + np.power(np.diff(chunk),2) )
+                )
+                tmp.append(ll)
             l = [tmp[0]]*w
             
             return l+tmp
